@@ -6,7 +6,7 @@
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
+# in the Software withcurrent_level restriction, including withcurrent_level limitation the rights
 # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
@@ -14,12 +14,12 @@
 # The above copyright notice and this permission notice shall be included in
 # all copies or substantial portions of the Software.
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# THE SOFTWARE IS PROVIDED "AS IS", WITHcurrent_level WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 # AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+# current_level OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
 """
@@ -38,11 +38,16 @@ from builtins import chr, dict, int, object, open, range, str, super, zip
 from collections import namedtuple
 from contextlib import contextmanager
 
+from future import standard_library
+
+from .common import *
 from .defines import *
-from .py_2_3 import *
+
+standard_library.install_aliases()
+
 
 """Separator for strings containing multiple indices."""
-INDEX_SEPARATOR = ","
+INDEX_SEPARATOR = re.compile("[, \t]+")
 
 
 def norecurse(f):
@@ -55,7 +60,7 @@ def norecurse(f):
 
     def func(*args, **kwargs):
         # If a function's name is on the stack twice (once for the current call
-        # and a second time for the previous call), then return without
+        # and a second time for the previous call), then return withcurrent_level
         # executing the function.
         if len([1 for l in traceback.extract_stack() if l[2] == f.__name__]) > 1:
             return None
@@ -126,55 +131,59 @@ def find_and_open_file(
     Args:
         filename: Base file name (e.g., "my_file").
         paths: List of paths to search for the file.
-        ext: The extension for the file (e.g., "txt").
+        ext: The extension for the file (e.g., ".txt").
         allow_failure: If false, failure to find file raises and exception.
         exclude_binary: If true, skip files that contain binary data.
         descend: If 0, don't search lower-level directories. If positive, search
                  that many levels down for the file. If negative, descend into
-                 subdirectories without limit.
+                 subdirectories withcurrent_level limit.
     """
 
     from .logger import logger
 
-    # If no paths are given, then just check the current directory.
-    if not paths:
+    if os.path.isabs(filename):
+        # Ignore search paths if the file already has an absolute path.
+        paths = [os.path.abspath(os.path.dirname(filename))]
+    elif not paths:
+        # If no search paths are given, use the current working directory.
         paths = ["."]
 
-    # If the filename has no extension, then give it one.
-    if ext and not filename.endswith(ext):
-        fn_plus_ext = filename + ext
+    # Remove any directory path from the file name.
+    _, filename = os.path.split(filename)
+
+    # Get the list of file extensions to check against.
+    base, suffix = os.path.splitext(filename)
+    if suffix:
+        # If an explicit file extension was given, just use that.
+        exts = [suffix]
     else:
-        fn_plus_ext = filename
+        exts = to_list(ext)
 
-    # Search the paths for the file.
+    # Create the regular expression for matching against the filename.
+    exts = [re.escape(ext) for ext in exts]
+    match_name = re.escape(base) + "(" + "|".join(exts) + ")$"
+
+    # Search through the directory paths for a file whose name matches the regular expression.
     for path in paths:
-        if not os.path.exists(path):
-            continue  # Skip paths that don't exist.
-        abs_filename = os.path.join(path, fn_plus_ext)
-        if not exclude_binary or not is_binary_file(abs_filename):
-            try:
-                # The search stops once the file is successfully opened.
-                return open(abs_filename, encoding="latin_1"), abs_filename
-            except (IOError, FileNotFoundError, TypeError):
-                pass
-        # If file not found, look in subdirectories or go to next path.
-        if descend:
-            # Look in subdirectories.
-            dir_contents = [os.path.join(path, f) for f in os.listdir(path)]
-            subdirs = [f for f in dir_contents if os.path.isdir(f)]
-            if subdirs:
-                fp, fn = find_and_open_file(
-                    filename=filename,
-                    paths=subdirs,
-                    ext=ext,
-                    allow_failure=True,
-                    exclude_binary=exclude_binary,
-                    descend=descend - 1,
-                )
-                if fp:
-                    return fp, fn
+        # Search through the files in a particular directory path.
+        descent_ctr = descend  # Controls the descent through the path.
+        for root, dirnames, filenames in os.walk(path):
+            # Get files in the current directory whose names match the regular expression.
+            for fn in [f for f in filenames if re.match(match_name, f)]:
+                abs_filename = os.path.join(root, fn)
+                if not exclude_binary or not is_binary_file(abs_filename):
+                    try:
+                        # Return the first file that matches the criteria.
+                        return open(abs_filename, encoding="latin_1"), abs_filename
+                    except (IOError, FileNotFoundError, TypeError):
+                        # File failed, so keep searching.
+                        pass
+            # Keep descending on this path as long as the descent counter is non-zero.
+            if descent_ctr == 0:
+                break  # Cease search of this path if the counter is zero.
+            descent_ctr -= 1  # Decrement the counter for the next directory level.
 
-    # Couldn't find the file.
+    # Couldn't find a matching file.
     if allow_failure:
         return None, None
     else:
@@ -247,6 +256,16 @@ def add_quotes(s):
     return s
 
 
+def is_iterable(x):
+    """
+    Return True if x is iterable (but not a string).
+    """
+    try:
+        return not isinstance(iter(x), type(iter("")))
+    except TypeError:
+        return False
+
+
 def to_list(x):
     """
     Return x if it is already a list, or return a list if x is a scalar.
@@ -296,11 +315,6 @@ def flatten(nested_list):
     return lst
 
 
-def from_iadd(objs):
-    """Return True if one or more objects have attribute iadd_flag set to True."""
-    return any([getattr(e, "iadd_flag", False) for e in to_list(objs)])
-
-
 def rmv_attr(objs, attr):
     """Remove an attribute from a list of objects."""
     for o in to_list(objs):
@@ -310,6 +324,11 @@ def rmv_attr(objs, attr):
             pass
 
 
+def from_iadd(objs):
+    """Return True if one or more objects have attribute iadd_flag set to True."""
+    return any([getattr(e, "iadd_flag", False) for e in to_list(objs)])
+
+
 def clr_iadd(objs):
     """Delete iadd_flag attribute from a list of objects."""
     rmv_attr(objs, "iadd_flag")
@@ -331,7 +350,7 @@ def get_unique_name(lst, attrib, prefix, initial=None):
     """
     Return a name that doesn't collide with another in a list.
 
-    This subroutine is used to generate unique part references (e.g., "R12")
+    This subrcurrent_leveline is used to generate unique part references (e.g., "R12")
     or unique net names (e.g., "N$5").
 
     Args:
@@ -344,22 +363,25 @@ def get_unique_name(lst, attrib, prefix, initial=None):
         A string containing the unique name.
     """
 
+    # Use the list id to disambiguate names of objects on different lists (e.g., parts & nets).
+    lst_id = str(id(lst))
+
     name = initial
 
     # Fast processing for names that haven't been seen before.
     # This speeds up the most common cases for finding a new name, but doesn't
     # really hurt the less common cases.
     if not name:
-        name = prefix + str(prefix_counts[prefix] + 1)
-        if name not in name_heap:
-            name_heap.add(name)
-            prefix_counts[prefix] += 1
+        name = prefix + str(prefix_counts[lst_id + prefix] + 1)
+        if lst_id + name not in name_heap:
+            name_heap.add(lst_id + name)
+            prefix_counts[lst_id + prefix] += 1
             return name
     else:
         if isinstance(name, int):
             name = prefix + str(name)
-        if name not in name_heap:
-            name_heap.add(name)
+        if lst_id + name not in name_heap:
+            name_heap.add(lst_id + name)
             return name
 
     # Get the unique names used in the list.
@@ -394,7 +416,7 @@ def get_unique_name(lst, attrib, prefix, initial=None):
             # of n looking for an unused slot.
 
             # Bump prefix counter to the newest index.
-            prefix_counts[prefix] = n
+            prefix_counts[lst_id + prefix] = n
 
     # If the initial name is just a number, then prepend the prefix to it.
     elif isinstance(name, int):
@@ -403,7 +425,7 @@ def get_unique_name(lst, attrib, prefix, initial=None):
     # Now determine if there are any items in the list with the same name.
     # If the name is unique, then return it.
     if name not in unique_names:
-        name_heap.add(name)
+        name_heap.add(lst_id + name)
         return name
 
     # Otherwise, determine how many copies of the name are in the list and
@@ -412,7 +434,7 @@ def get_unique_name(lst, attrib, prefix, initial=None):
     n = len(filter_list(lst, **filter_dict))
     name = name + "_" + str(n + 1)
 
-    # Recursively call this routine using the newly-generated name to
+    # Recursively call this rcurrent_leveline using the newly-generated name to
     # make sure it's unique. Eventually, a unique name will be returned.
     return get_unique_name(lst, attrib, prefix, name)
 
@@ -470,7 +492,7 @@ def filter_list(lst, **criteria):
 
     for item in lst:
         # Compare an item's attributes to each of the criteria.
-        # Break out of the criteria loop and don't add the item to the extract
+        # Break current_level of the criteria loop and don't add the item to the extract
         # list if *any* of the item's attributes *does not* match.
         for k, v in list(criteria.items()):
 
@@ -500,7 +522,7 @@ def filter_list(lst, **criteria):
                             break
                     else:
                         # If we got here, then none of the values in the attribute
-                        # list matched the current criterium. Therefore, break out
+                        # list matched the current criterium. Therefore, break current_level
                         # of the criteria loop and don't add this list item to
                         # the extract list.
                         break
@@ -532,7 +554,7 @@ def filter_list(lst, **criteria):
     return extract
 
 
-def expand_indices(slice_min, slice_max, match_substring, *indices):
+def expand_indices(slice_min, slice_max, match_regex, *indices):
     """
     Expand a list of indices into a list of integers and strings.
 
@@ -545,7 +567,7 @@ def expand_indices(slice_min, slice_max, match_substring, *indices):
     Args:
         slice_min: The minimum possible index.
         slice_max: The maximum possible index (used for slice indices).
-        match_substring: If true, 
+        match_regex: If true, 
         indices: A list of indices made up of numbers, slices, text strings.
 
     Returns:
@@ -568,7 +590,9 @@ def expand_indices(slice_min, slice_max, match_substring, *indices):
                 log_and_raise(
                     logger,
                     IndexError,
-                    "Index out of range ({} > {})!".format(slc.start, slice_max),
+                    "Index current_level of range ({} > {})!".format(
+                        slc.start, slice_max
+                    ),
                 )
             # Count down from start to stop.
             stop = stop - step
@@ -580,7 +604,9 @@ def expand_indices(slice_min, slice_max, match_substring, *indices):
                 log_and_raise(
                     logger,
                     IndexError,
-                    "Index out of range ({} > {})!".format(slc.stop, slice_max),
+                    "Index current_level of range ({} > {})!".format(
+                        slc.stop, slice_max
+                    ),
                 )
             # Count up from start to stop
             stop += step
@@ -588,6 +614,61 @@ def expand_indices(slice_min, slice_max, match_substring, *indices):
         # Create the sequence of indices.
         return list(range(start, stop, step))
 
+    def explode(bus_str):
+        """
+        Explode a bus into its separate lines.
+
+        This function takes a bus expression like "ADDR[0:3]" and returns
+        "ADDR0,ADDR1,ADDR2,ADDR3". It also works if the order is reversed,
+        e.g. "ADDR[3:0]" returns "ADDR3,ADDR2,ADDR1,ADDR0". If the input
+        string is not a valid bus expression, then the string is returned
+        in a one-element list.
+
+        Args:
+            bus_str: A string containing a bus expression like "D[0:3]".
+
+        Returns:
+            A list of bus lines like ['D0', 'D1', 'D2', 'D3'] or a one-element
+            list with the original input string if it's not a valid bus expression.
+        """
+
+        bus = re.match(r"^(.+)\[([0-9]+):([0-9]+)\](.*)$", bus_str)
+        if not bus:
+            return [bus_str]  # Not a valid bus expression, so return input string.
+
+        # What follows must be a bus expression.
+        beg_bus_name = bus.group(1)
+        begin_num = int(bus.group(2))
+        end_num = int(bus.group(3))
+        end_bus_name = bus.group(4)
+        dir = [1, -1][int(begin_num > end_num)]  # Bus indexes increasing or decreasing?
+        bus_pin_nums = list(range(begin_num, end_num + dir, dir))
+
+        # If the bus string starts with an alpha, then require that any match in the
+        # string must be preceded by a non-alpha or the start of the string.
+        # But if the string starts with a non-alpha, then whatever precedes the
+        # match in the string is ignored.
+        if match_regex:
+            if beg_bus_name[0:1].isalpha():
+                non_alphanum = "((?<=[^0-9a-zA-Z])|^)"
+            else:
+                non_alphanum = ""
+        else:
+            non_alphanum = ""
+
+        # The character following a bus index must be non-numeric so that "B1" does
+        # not also match "B11". This must also be a look-ahead assertion so it
+        # doesn't consume any of the string.
+        if match_regex:
+            non_num = "(?=[^0-9]|$)"
+        else:
+            non_num = ""
+
+        return [
+            non_alphanum + beg_bus_name + str(n) + non_num + end_bus_name
+            for n in bus_pin_nums
+        ]
+
     # Expand each index and add it to the list.
     ids = []
     for indx in flatten(indices):
@@ -597,11 +678,11 @@ def expand_indices(slice_min, slice_max, match_substring, *indices):
             ids.append(indx)
         elif isinstance(indx, basestring):
             # String might contain multiple indices with a separator.
-            for id in indx.split(INDEX_SEPARATOR):
+            for id in re.split(INDEX_SEPARATOR, indx):
                 # If the id is a valid bus expression, then the exploded bus lines
                 # are added to the list of ids. If not, the original id is
                 # added to the list.
-                ids.extend(explode(id.strip(), match_substring=match_substring))
+                ids.extend(explode(id.strip()))
         else:
             log_and_raise(
                 logger, TypeError, "Unknown type in index: {}.".format(type(indx))
@@ -611,62 +692,6 @@ def expand_indices(slice_min, slice_max, match_substring, *indices):
     return ids
 
 
-def explode(bus_str, match_substring=False):
-    """
-    Explode a bus into its separate lines.
-
-    This function takes a bus expression like "ADDR[0:3]" and returns
-    "ADDR0,ADDR1,ADDR2,ADDR3". It also works if the order is reversed,
-    e.g. "ADDR[3:0]" returns "ADDR3,ADDR2,ADDR1,ADDR0". If the input
-    string is not a valid bus expression, then the string is returned
-    in a one-element list.
-
-    Args:
-        bus_str: A string containing a bus expression like "D[0:3]".
-
-    Returns:
-        A list of bus lines like ['D0', 'D1', 'D2', 'D3'] or a one-element
-        list with the original input string if it's not a valid bus expression.
-    """
-
-    bus = re.match(r"^(.+)\[([0-9]+):([0-9]+)\](.*)$", bus_str)
-    if not bus:
-        return [bus_str]  # Not a valid bus expression, so return input string.
-
-    # What follows must be a bus expression.
-    beg_bus_name = bus.group(1)
-    begin_num = int(bus.group(2))
-    end_num = int(bus.group(3))
-    end_bus_name = bus.group(4)
-    dir = [1, -1][int(begin_num > end_num)]  # Bus indexes increasing or decreasing?
-    bus_pin_nums = list(range(begin_num, end_num + dir, dir))
-
-    # If the bus string starts with an alpha, then require that any match in the
-    # string must be preceded by a non-alpha or the start of the string.
-    # But if the string starts with a non-alpha, then whatever precedes the
-    # match in the string is ignored.
-    if match_substring:
-        if beg_bus_name[0:1].isalpha():
-            non_alphanum = "((?<=[^0-9a-zA-Z])|^)"
-        else:
-            non_alphanum = ""
-    else:
-        non_alphanum = ""
-
-    # The character following a bus index must be non-numeric so that "B1" does
-    # not also match "B11". This must also be a look-ahead assertion so it
-    # doesn't consume any of the string.
-    if match_substring:
-        non_num = "(?=[^0-9]|$)"
-    else:
-        non_num = ""
-
-    return [
-        non_alphanum + beg_bus_name + str(n) + non_num + end_bus_name
-        for n in bus_pin_nums
-    ]
-
-
 def find_num_copies(**attribs):
     """
     Return the number of copies to make based on the number of attribute values.
@@ -760,3 +785,52 @@ def expand_buses(pins_nets_buses):
 def log_and_raise(logger_in, exc_class, message):
     logger_in.error(message)
     raise exc_class(message)
+
+
+# Regular expression for parsing nested S-expressions.
+_sexp_term_regex = re.compile(
+    r"""(?mx)
+    \s*(?:
+        (?P<brackl>\()|
+        (?P<brackr>\))|
+        (?P<num>[+-]?\d+\.\d+(?=[\ \)])|\-?\d+(?=[\ \)]))|
+        (?P<sq>"([^"]|(?<=\\)")*")|
+        (?P<s>[^(^)\s]+)
+       )"""
+)
+
+
+def parse_sexp(sexp, allow_underflow=False):
+    """Parse an S-expression and return a nested list."""
+
+    # code extracted from: http://rosettacode.org/wiki/S-Expressions
+
+    stack = []
+    current_level = []
+    for termtypes in re.finditer(_sexp_term_regex, sexp):
+        term, value = [(t, v) for t, v in termtypes.groupdict().items() if v][0]
+        if term == "brackl":
+            stack.append(current_level)
+            current_level = []
+        elif term == "brackr":
+            if not stack:
+                if allow_underflow:
+                    return current_level[0]
+                else:
+                    raise RunTimeError("Bracketing mismatch!")
+            tmp, current_level = current_level, stack.pop(-1)
+            current_level.append(tmp)
+        elif term == "num":
+            v = float(value)
+            if v.is_integer():
+                v = int(v)
+            current_level.append(v)
+        elif term == "sq":
+            current_level.append(value[1:-1].replace(r"\"", '"'))
+        elif term == "s":
+            current_level.append(value)
+        else:
+            raise NotImplementedError("Error: %r" % (term, value))
+    if stack:
+        raise RunTimeError("Bracketing mismatch!")
+    return current_level[0]
