@@ -59,3 +59,16 @@ def test_name_5():
     for _ in range(l):
         n = Net()
     assert len(default_circuit.nets) == l + k + 1  # Account for NC net.
+
+def test_name_6():
+    # Test that net and part naming don't affect each other.
+    net_r1 = Net('R1')
+    r1 = Part('Device', 'R')
+    r2 = Part('Device', 'R')
+    assert net_r1.name == 'R1'
+    assert r1.ref == 'R1'
+    assert r2.ref == 'R2'
+    r3 = Part('Device', 'R', ref='R2')
+    assert r3.ref == 'R2_1'
+    net_r2 = Net('R1')
+    assert net_r2.name == 'R1_1'
